#include "Core.h"
#include <string>
#include <fstream>
#include <iomanip>
#include <sys/time.h>

Core::Core()
{ 
  timeval the_time;
  gettimeofday(&the_time,NULL);
  random.seed((long int)the_time.tv_usec);
}

void Core::fire_net()
{
  for( int i = 0; i < NETSIZE; i++ )    
    net[i]=logic(*net_input[i][0],*net_input[i][1]);
  for( int i = 0; i < SHOWSIZE; i++ )   
    show[i]=logic(*show_input[i][0],*show_input[i][1]);
}

Core::~Core()
{}

char Core::get()
{
  int get_from;
  for( int i = 0; i < 8; i++ )
    {
      get_from = (index+i);
      get_from %=SHOWSIZE;
      if( get_from == 0 ) fire_net();
      rval[i] = show[get_from];
    }
  index = index+8;
  index%=SHOWSIZE;
  return (char) rval.to_ulong();
}
bool Core::logic( const bool& one, const bool& two )
{
  return !(one^two);
}
void Core::core_store_key( const char* in )
{
  index=0;
  int w1,w2;
  fstream out( in, ios::out );
  out<<NETSIZE<<" "<<SHOWSIZE<<endl;
  out<<"<NEURONS>"<<endl;
  for( int i = 0; i < NETSIZE; i++ )
    {
      // logic network
      w1=random(NETSIZE-1);
      w2=random(NETSIZE-1);
      // remove cyclic sets, can't connect to self
      if( w1>=i )
	w1++;
      if( w2>=i )
	w2++;
      while( w1 == w2 )
	{
	  w1=random(NETSIZE-1);
	  w2=random(NETSIZE-1);
	  if( w1>=i )
	    w1++;
	  if( w2>=i )
	    w2++;
	}
      net_input[i][0]=&net[w1];
      net_input[i][1]=&net[w2];
      out<<w1<<" "<<w2<<endl;
    }
  out<<"<SHOW>"<<endl;
  for( int i = 0; i < SHOWSIZE; i++ )
    {
      w1=random(NETSIZE);
      w2=random(NETSIZE);
      while( w1 == w2 )
	{
	  w1=random(NETSIZE);
	  w2=random(NETSIZE);
	}
      show_input[i][0]=&net[w1];
      show_input[i][1]=&net[w2];
      out<<w1<<" "<<w2<<endl;
    }
  fire_net();
}

void Core::core_get_key( const char* in_name )
{
  fstream in(in_name, ios::in );
  string read;
  in>>read;
  if( atoi(read.c_str()) != NETSIZE )
    {
      cerr<<"core_get_key != NETSIZE"<<endl;
      throw -1;
    }
  in>>read;
  if( atoi(read.c_str()) != SHOWSIZE )
    {
      cerr<<"core_get_key != SHOWSIZE"<<endl;
      throw -1;
    }
  in>>read;
  if( read != "<NEURONS>" )
    {
      cerr<<"core_get_key != <NEURONS>"<<endl;
      throw -1;
    }
  int one,two;
  for( int i =0; ; i++ )
    {
      in>>read;
      if( read == "<SHOW>" )
	break;
      one=atoi(read.c_str());
      in>>read;
      two=atoi(read.c_str());
      net_input[i][0]=&net[one];
      net_input[i][1]=&net[two];
    }
  for( int i =0; ; i++ )
    {
      in>>read;
      if( !in.good() )
	break;
      one=atoi(read.c_str());
      in>>read;
      two=atoi(read.c_str());
      show_input[i][0]=&net[one];
      show_input[i][1]=&net[two];
    }
  fire_net();
}

